/**
 * Soluzione che risponde alle query in O(N+Q) usando LCA offline (versione
 * pulita)
 */
#include <bits/stdc++.h>
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream& in(cin);
ostream& out(cout);
#endif

struct Edge {
    int to,    // nodo di destinazione
        index; // indice dell'arco
    Edge(int _to, int _index) : to(_to), index(_index) {}
};

struct MFset {
    vector<int> parent, rank;

    void init(int n) {
        rank.resize(n, 0);
        parent.resize(n);
        for (int i = 0; i < n; i++)
            parent[i] = i;
    }

    int find(int x) { return parent[x] != x ? parent[x] = find(parent[x]) : x; }

    void merge(int x, int y) {
        int px = find(x), py = find(y);
        if (px != py) {
            if (rank[px] < rank[py]) {
                parent[px] = py;
            } else {
                parent[py] = px;
                rank[px] += rank[px] == rank[py];
            }
        }
    }
};

struct Tree {
    vector<vector<Edge>> tree;
    vector<bool> visited;
    vector<int> count_a, // quante volte il nodo i è source
        count_b,         // quante volte il nodo i è dest
        count_c;         // quante volte il nodo i è lca
    vector<int> ancestor;

    MFset lca_set;

    void init(int n) { tree.resize(n); }

    void add_undirected_edge(int a, int b, int id) {
        tree[a].push_back(Edge(b, id));
        tree[b].push_back(Edge(a, id));
    }

    /**
     * Orienta gli archi dell'albero e trova un eventuale arco in conflitto.
     */
    bool dfs_orient_edges(int v, int parent, vector<vector<int>>& node_queries,
                          vector<pair<int, int>>& edges_orientation,
                          int& conflict_edge) {
        ancestor[v] = v;
        int parent_edge = -1;
        for (Edge& e : tree[v]) {
            int u = e.to;
            if (u != parent) {
                if (!dfs_orient_edges(u, v, node_queries, edges_orientation,
                                      conflict_edge))
                    return false;
                lca_set.merge(v, u);
                ancestor[lca_set.find(v)] = v;
                count_a[v] += count_a[u];
                count_b[v] += count_b[u];
                count_c[v] += count_c[u];
            } else
                parent_edge = e.index;
        }

        visited[v] = true;
        for (int other_node : node_queries[v])
            if (visited[other_node])
                count_c[ancestor[lca_set.find(other_node)]]++;

        // oriento gli archi
        if (count_a[v] > count_c[v] && count_b[v] > count_c[v]) {
            assert(parent_edge != -1);
            conflict_edge = parent_edge;
            return false;
        }
        assert(count_a[v] >= count_c[v] && count_b[v] >= count_c[v]);
        if (parent_edge != -1) {
            if (count_a[v] > count_c[v])
                edges_orientation[parent_edge] = make_pair(v, parent);
            else if (count_b[v] > count_c[v])
                edges_orientation[parent_edge] = make_pair(parent, v);
            else
                assert(count_a[v] == count_b[v] && count_b[v] == count_c[v]);
        }
        return true;
    }

    /**
     * Orienta gli archi dell'albero per rispondere alle query
     * Source LCA offline:
     * https://en.wikipedia.org/wiki/Tarjan%27s_off-line_lowest_common_ancestors_algorithm
     */
    bool answer_queries(vector<pair<int, int>>& queries,
                        vector<pair<int, int>>& edges_orientation,
                        int& conflict_edge) {
        vector<vector<int>> node_queries(tree.size());
        count_a.assign(tree.size(), 0);
        count_b.assign(tree.size(), 0);
        count_c.assign(tree.size(), 0);
        for (auto& q : queries) {
            if (q.first != q.second) {
                node_queries[q.first].push_back(q.second);
                node_queries[q.second].push_back(q.first);
                count_a[q.first]++;
                count_b[q.second]++;
            }
        }

        lca_set.init(tree.size());
        ancestor.assign(tree.size(), -1);
        visited.assign(tree.size(), false);
        return dfs_orient_edges(0, -1, node_queries, edges_orientation,
                                conflict_edge);
    }
};

struct Graph {
    vector<vector<Edge>> g;
    vector<bool> visited;
    vector<int> bfc;
    vector<bool> is_bridge;
    vector<int> bridges;
    vector<pair<int, int>> edges;

    Graph(int n) : g(n) {}

    void add_undirected_edge(int a, int b) {
        g[a].push_back(Edge(b, edges.size()));
        g[b].push_back(Edge(a, edges.size()));
        is_bridge.push_back(false);
        edges.push_back(make_pair(a, b));
    }

    void dfs_bridges(int v, vector<int>& tin, vector<int>& low, int& timer,
                     int p = -1) {
        visited[v] = true;
        tin[v] = low[v] = timer++;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (to == p)
                continue;
            if (visited[to]) {
                low[v] = min(low[v], tin[to]);
            } else {
                dfs_bridges(to, tin, low, timer, v);
                low[v] = min(low[v], low[to]);
                // assert(edges_count[make_pair(min(v, to), max(v, to))] >= 1);
                if (low[to] > tin[v]) {
                    is_bridge[e.index] = true;
                    bridges.push_back(e.index);
                }
            }
        }
    }

    /**
     * Source: https://cp-algorithms.com/graph/bridge-searching.html
     */
    void find_bridges() {
        visited.assign(g.size(), false);
        vector<int> tin(g.size(), -1), low(g.size(), -1);
        int timer = 0;
        for (int i = 0; i < g.size(); ++i) {
            if (!visited[i])
                dfs_bridges(i, tin, low, timer);
        }
    }

    /**
     * Visito la bridge-free-component
     */
    void visit_bfc(int v, int counter) {
        bfc[v] = counter;
        // cerr << "BFC[" << v << "] = " << counter << endl;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (bfc[to] == -1 && !is_bridge[e.index])
                visit_bfc(to, counter);
        }
    }

    /**
     * Costruisce il bridge tree del grafo
     */
    void get_bridge_tree(Tree& t) {
        find_bridges();
        int counter = 0;
        bfc.assign(g.size(), -1);
        for (int i = 0; i < g.size(); i++)
            if (bfc[i] == -1)
                visit_bfc(i, counter++);
        t.init(counter);
        for (int i = 0; i < bridges.size(); i++) {
            int b = bridges[i];
            // assert(bfc[edges[b].first] != bfc[edges[b].second]);
            t.add_undirected_edge(bfc[edges[b].first], bfc[edges[b].second], i);
        }
    }
};

/**
 * Data un'orientazione degli archi del bridge tree, orienta i bridge
 * corrispondenti
 */
void get_bridges_orientation(vector<pair<int, int>>& tree_edges, Graph& g,
                             vector<pair<int, int>>& oriented_bridges) {

    for (int i = 0; i < g.bridges.size(); i++) {
        pair<int, int>& graph_edge = g.edges[g.bridges[i]];
        pair<int, int>& tree_edge = tree_edges[i];
        if (tree_edge != make_pair(-1, -1)) {
            if (tree_edge.first != g.bfc[graph_edge.first])
                swap(graph_edge.first, graph_edge.second);
            assert(tree_edge == make_pair(g.bfc[graph_edge.first],
                                          g.bfc[graph_edge.second]));
            oriented_bridges.push_back(graph_edge);
        }
    }
}

/**
 * Traduce query sul grafo originale in query sul bridge tree
 */
void translate_queries(vector<pair<int, int>>& queries, vector<int>& bfc) {
    for (auto& q : queries) {
        q.first = bfc[q.first];
        q.second = bfc[q.second];
    }
}

int main() {
    int n, m, q;
    int a, b;
    in >> n >> m >> q;
    Graph g(n);
    for (int i = 0; i < m; i++) {
        in >> a >> b;
        g.add_undirected_edge(a, b);
    }
    vector<pair<int, int>> queries;
    for (int i = 0; i < q; i++) {
        in >> a >> b;
        queries.push_back(make_pair(a, b));
    }
    // costruisco il bridge tree
    Tree bridge_tree;
    g.get_bridge_tree(bridge_tree);
    // trovo un'orientazione agli archi dell'albero
    vector<pair<int, int>> tree_edges_orientation(g.bridges.size(),
                                                  make_pair(-1, -1));
    int conflict_edge = -1;
    translate_queries(queries, g.bfc);

    if (bridge_tree.answer_queries(queries, tree_edges_orientation,
                                   conflict_edge)) {
        out << "1" << endl;
        // riconverto gli archi dell'albero nei bridges originali
        vector<pair<int, int>> oriented_bridges;
        get_bridges_orientation(tree_edges_orientation, g, oriented_bridges);
        out << oriented_bridges.size() << endl;
        for (auto& bridge : oriented_bridges)
            out << bridge.first << " " << bridge.second << endl;

    } else {
        out << "0" << endl;
        pair<int, int> conflict = g.edges[g.bridges[conflict_edge]];
        out << conflict.first << " " << conflict.second << endl;
    }
}
